
/*****************************RC INCIDENCE METER************************************
                                Max Kaiser   2018

V 1.2 07.01.2019 
fixed error where resetting didnt work a second Time. Thx to Georg!

V 1.1

Main components:
2 pc Accelerometer GY-45 MMA8451
1 pc Arduino Nano 328
1 pc 16*2 LCD Display
1 pc I²C-Interface for LCD
3D printed parts    https://www.thingiverse.com/thing:2938217

Gehe zu https://www.rc-maker.com für weitere Details

Eine kommerzielle Nutzung ist nicht erlaubt!
Commercial use is prohibited!

/************************************ SETTINGS ************************************/                                            
bool debugging = false;     //Set to true for debugging via serial monitor
unsigned long lcdRefreshRate = 500;     //lcd refresh rate. set to desired rate in milliseconds


/*********************************CALIBRATION VALUES*******************************/
/*****************!!RUN THE CALIBRATION SKETCH AND FILL IN YOUR OWN VALUES!!****************/

#define offset_AX   0       // OFFSET values 
#define offset_AY   0    
#define offset_AZ   0

#define offset_BX   0       // OFFSET values
#define offset_BY   0
#define offset_BZ   0

const double gain_AX = 1;     // GAIN factors
const double gain_AY = 1;
const double gain_AZ = 1;  

#define gain_BX     1     // GAIN factors
#define gain_BY     1
#define gain_BZ     1

/************************************ INCLUDES ********************************************/

#include <Arduino.h>
#include "Wire.h"
#include <Adafruit_MMA8451.h>       //https://github.com/adafruit/Adafruit_MMA8451_Library
#include <Adafruit_Sensor.h>        //https://github.com/adafruit/Adafruit_Sensor
#include <LCD.h>        //https://bitbucket.org/fmalpartida/new-liquidcrystal/wiki/Home
#include <LiquidCrystal_I2C.h>      //https://bitbucket.org/fmalpartida/new-liquidcrystal/wiki/Home
#include <Filters.h>        //https://github.com/JonHub/Filters
#include <OneButton.h>      //https://github.com/mathertel/OneButton

/************************************ DEFINED VARIABLES ************************************/

float raw_AX, raw_AY, raw_AZ = 0.0; //raw data from sensor A
float raw_BX, raw_BY, raw_BZ = 0.0; //raw data from sensor B

float g_value_AX, g_value_AY, g_value_AZ = 0.0; //g-values from angle sensor A
float g_value_BX, g_value_BY, g_value_BZ = 0.0; //g-values from angle sensor B

float deg_SensorA = 0.0; //Holds the sensor value in degrees
float deg_SensorB = 0.0; //Holds the sensor value in degrees

float deg_RelativeA = 0.0; //FOR ZEROING
float deg_RelativeB = 0.0; //FOR ZEROING

float filterFrequency = 0.4;    //filters out changes faster than 0.4 Hz
int average;    //counter for averaging

bool lcdText = false;      //is true if text on lcd is already written to avoid flicker

unsigned long startMillis = 0;  //
unsigned long actualMillis  = 0;

/*********************************************************************************************/

LiquidCrystal_I2C lcd(0x3F,2,1,0,4,5,6,7);  //create a object of LiquidCrystal 
Adafruit_MMA8451 sensor_A = Adafruit_MMA8451(); //create a sensor object
Adafruit_MMA8451 sensor_B = Adafruit_MMA8451(); //create a sensor object

FilterOnePole lowpassFilter_AX( LOWPASS, filterFrequency );    // create a one pole (RC) lowpass filter 
FilterOnePole lowpassFilter_AY( LOWPASS, filterFrequency );    // create a one pole (RC) lowpass filter
FilterOnePole lowpassFilter_AZ( LOWPASS, filterFrequency );    // create a one pole (RC) lowpass filter

FilterOnePole lowpassFilter_BX( LOWPASS, filterFrequency );    // create a one pole (RC) lowpass filter 
FilterOnePole lowpassFilter_BY( LOWPASS, filterFrequency );    // create a one pole (RC) lowpass filter
FilterOnePole lowpassFilter_BZ( LOWPASS, filterFrequency );    // create a one pole (RC) lowpass filter

OneButton button(5,true);   //Instance of OneButton which is used to test if button is pressed

/************************************ DEFINED FUNCTIONS ************************************/

//This function will run once at startup
void setup() {

    startMillis = millis();     //used for lcd timer

    //initialice LCD
    lcd.begin(16, 2);
    lcd.setBacklightPin(3,POSITIVE);
    lcd.setBacklight(HIGH);
    lcd.clear();

    if(debugging==true) Serial.begin(9600);

    //check an initialize sensors
    if (!sensor_A.begin(0x1C))
    {
        lcd.setCursor(0,0);
        lcd.print("Sensor A failed");
        while (1);
    }//end if

    sensor_A.setRange(MMA8451_RANGE_2_G);
    if (debugging==true){
        Serial.println("Sensor_A found!");
        Serial.print("Range A = ");
        Serial.print(2 << sensor_A.getRange());
        Serial.println("G");
    }//end if
    lcd.setCursor(0,0);
    lcd.print("Sensor A OK");


    if (!sensor_B.begin(0x1D))
    {
        lcd.setCursor(0,1);
        lcd.print("Sensor B failed");
        while (1);
    }//end if

    sensor_B.setRange(MMA8451_RANGE_2_G);
    if(debugging==true){
        Serial.println("Sensor_B found!");
        Serial.print("Range B = ");
        Serial.print(2 << sensor_B.getRange());
        Serial.println("G");
    }//end if
    lcd.setCursor(0,1);
    lcd.print("Sensor B OK");

    //call zeroing function if button is pressed for longer time
    button.attachLongPressStart(zeroingFunction);
    //wait for 2 seconds
    delay(2000);
    //clear screen
    lcd.clear();
}//end setup

//this function runs repeatedly
void loop(){
    button.tick();    //test if button is pressed
    actualMillis = millis();
    
    sensor_A.read();
    raw_AX = ((float) sensor_A.x - offset_AX) / gain_AX;    //compute offset from calibration
    raw_AY = ((float) sensor_A.y - offset_AY) / gain_AY;    //compute offset from calibration
    raw_AZ = ((float) sensor_A.z - offset_AZ) / gain_AZ;    //compute offset from calibration
    
    lowpassFilter_AX.input(raw_AX);    //filter sensor noise
    lowpassFilter_AY.input(raw_AY);    //filter sensor noise
    lowpassFilter_AZ.input(raw_AZ);    //filter sensor noise

    g_value_AX += raw_AX;       //add read value to variable 
    g_value_AY += raw_AY;       //add read value to variable
    g_value_AZ += raw_AZ;       //add read value to variable
        
    sensor_B.read();
    raw_BX = ((float) sensor_B.x - offset_BX) / gain_BX;    //compute offset from calibration
    raw_BY = ((float) sensor_B.y - offset_BY) / gain_BY;    //compute offset from calibration
    raw_BZ = ((float) sensor_B.z - offset_BZ) / gain_BZ;    //compute offset from calibration
    
    lowpassFilter_BX.input(raw_BX);    //filter sensor noise
    lowpassFilter_BY.input(raw_BY);    //filter sensor noise
    lowpassFilter_BZ.input(raw_BZ);    //filter sensor noise

    g_value_BX += raw_BX;       //add read value to variable 
    g_value_BY += raw_BY;       //add read value to variable
    g_value_BZ += raw_BZ;       //add read value to variable
    
    average++;      //add 1 to counter


    //test if lcdRefreshRate is elapsed
    if(actualMillis-startMillis >= lcdRefreshRate){
        g_value_AX = (g_value_AX / average) * (1.00 / 4095.00);     //build average and convert to G ( 1G = gravity)
        g_value_AY = (g_value_AY / average) * (1.00 / 4095.00);
        g_value_AZ = (g_value_AZ / average) * (1.00 / 4095.00);

        g_value_BX = (g_value_BX / average) * (1.00 / 4095.00);     //build average and convert to G ( 1G = gravity)
        g_value_BY = (g_value_BY / average) * (1.00 / 4095.00);
        g_value_BZ = (g_value_BZ / average) * (1.00 / 4095.00); 

        deg_SensorA = (atan2(g_value_AY, (sqrt(g_value_AX * g_value_AX + g_value_AZ * g_value_AZ))) * 180.0) / PI;      //compute angle out of the 3 Axis und convert to degrees
        deg_SensorB = (atan2(g_value_BY, (sqrt(g_value_BX * g_value_BX + g_value_BZ * g_value_BZ))) * 180.0) / PI;      //compute angle out of the 3 Axis und convert to degrees
        
        deg_SensorA -= deg_RelativeA; //zeroing
        deg_SensorB -= deg_RelativeB; //zeroing
        
        printToLcd(deg_SensorA, deg_SensorB, deg_SensorA - deg_SensorB);   //print values to 16/2 LCD
        average = 0;
        startMillis = millis();     //reset lcd timer
    }// end if
}//end loop

// This function will be called once, when the button is pressed for a long time. It will set the actual value
//of the sensors to a variable used for zeroing
void zeroingFunction() {
     deg_RelativeA = (atan2(g_value_AY, (sqrt(g_value_AX * g_value_AX + g_value_AZ * g_value_AZ))) * 180.0) / PI;      //compute angle out of the 3 Axis and convert to degrees, set angle offset
     deg_RelativeB = (atan2(g_value_BY, (sqrt(g_value_BX * g_value_BX + g_value_BZ * g_value_BZ))) * 180.0) / PI;      //compute angle out of the 3 Axis and convert to degrees, set angle offset
     lcd.clear();
     lcd.setCursor(0,0);
     lcd.print("Reset OK!");
     delay(2000 );
     lcdText = false;
} //end zeroingFunction

//This function will format our values and write them to the LCD
void printToLcd(double deg_wing, double deg_elevator, double deg_horDihedral) {
    //check if Text on LCD is already written. If so, dont write again to avoid flicker
    if(lcdText==false){
        //clear lcd and write text
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("A:");
        lcd.setCursor(8, 0);
        lcd.print("B:");
        lcd.setCursor(2, 1);
        lcd.print("A-B:");
        lcdText = true;
    }
    
    char bufferWing[18];        
    dtostrf(deg_wing, 4, 1, bufferWing);        //convert double and store in char
    lcd.setCursor(2, 0);
    lcd.print(bufferWing);
    lcd.setCursor(6, 0);
    lcd.print((char)223);       //print deg symbol
    lcd.print(" ");
 
    
    char bufferElevator[18];
    dtostrf(deg_elevator, 4, 1, bufferElevator);        //convert double and store in char
    lcd.setCursor(10, 0);
    lcd.print(bufferElevator);
    lcd.setCursor(14, 0);
    lcd.print((char)223);       //print deg symbol
          
    char buffer_horDihedral[18];
    dtostrf(deg_horDihedral, 5, 1, buffer_horDihedral);     //convert double and store in char
    lcd.setCursor(6, 1);
    lcd.print(buffer_horDihedral);
    lcd.setCursor(11, 1);
    lcd.print((char)223);       //print deg symbol

}//end printToLcd